classdef LiftingSurface
    %LIFTINGSURFACE Implimentation of Multhopps lifting surface methadology
    %   Detailed theory can be found at http://naca.central.cranfield.ac.uk/reports/arc/rm/2884.pdf
    
    properties (SetAccess = private, GetAccess = public)
        m
        span
        AR
        sweep
        taper
        MAC
        v
        eta
        Cr
        theta
        Cv
        S
        calc
        sol
        coefs
        
        beta = 0;
        beta_rot
        beta0Aft;
        beta_span;
        beta_cen;
        
        c25
        
        ctrl
        
        CG = [0;0;0];
        
        dad
        dadd
        
        dCls
        dCms
    end
    
    methods
        function obj = LiftingSurface(nPoints)
            %LIFTINGSURFACE Construct an instance of this class
            %   Intialise class with odd number of points
            
            %There must be an odd number of points
            if mod(nPoints,2) == 0
                nPoints = nPoints+1;
            end
            obj.m = nPoints;
        end
        
        function obj = setCG(obj, CG)
            if length(CG) == 1
                obj.CG(1) = CG;
            else
                obj.CG = CG;
            end
            
            if obj.beta ~=0
                obj.CG([2 1]) = obj.beta_rot*obj.CG([2 1]) - [obj.beta_cen; 0];
            end
        end
        
        function obj = setGeometry(obj, span, AR, sweep, taper, beta)
            %setGeometry Specify geometry and calculate influence matrix
            
            %Re-initialise class to avoid conflicts when changing beta
            obj = LiftingSurface(obj.m);
            
            %Number of spanwise stations must be at least 3 times the
            %aspect ratio
            if obj.m < AR*3
                obj = LiftingSurface(AR*3);
            end
            
            m = obj.m;
            
            %Calculate wing area from span and aspect ratio
            S = span^2/AR;
            
            %Calculate chord at root and tip
            Cr = (2*S)/(span*(taper+1));
            Ct = Cr*taper;
            
            %Spanwise stations at eta specified by cos(theta)
            %Where theta is eevenly spaced between pi and 0

            %Evenly spaced thatea
            v = -0.5*(m-1):0.5*(m-1);
            theta_v = 0.5*pi - (v.*pi)./(m+1);

            
            theta_vf = [theta_v(1) + pi/(m+1) theta_v theta_v(end) - pi/(m+1)];

            %Transform from angle to non-dimensional spanwise location
            % eta = 2y/b
            eta = cos(theta_v);
            etaf = cos(theta_vf);

            %Dimensional location of each station
            y_v = eta.*0.5*span;
            
             % Calculate chord for each section

            C_v = interp1([-1, 0, 1], [Ct, Cr, Ct], eta, 'linear');
            

            % Calculte x0,l (leading edge X)

            x0_l = abs(y_v.*tan(deg2rad(sweep))) + 0.25*Cr - 0.25.*C_v;
            
            %Rotate geometry if beta is specified
            if nargin == 6
                beta = -beta;
                obj.beta = deg2rad(beta);
                
                aft = x0_l+C_v;
                
                %Rotate Geometry;
                r = [cosd(beta) -sind(beta); sind(beta) cosd(beta)];
                
                obj.beta_rot = r;
                
                fore = [y_v; x0_l];
                aft = [y_v; aft];
                
                obj.beta0Aft = aft;
                
                fore = r*fore;   
                aft = r*aft;
                
                cen = mean([min([fore(1,:) aft(1,:)]), max([fore(1,:) aft(1,:)])]);
                
                fore(1,:) = fore(1,:) - cen;
                aft(1,:) = aft(1,:) - cen;
                
                
               obj.beta_span = 2*max([fore(1,:) aft(1,:)]);
               obj.beta_cen = cen;
                
                                
                if beta > 0
                    fore = [aft(:,1) fore];
                    aft = [aft fore(:,end)];
                elseif beta ~= 0
                    fore = [fore aft(:,end)];
                    aft = [fore(:,1) aft];
                end
                    
                
                y_v = eta.*max([fore(1,:) aft(1,:)]);
                x0_l = interp1(fore(1,:), fore(2,:), y_v, 'linear');
                               
                   
                
            end

            % Update central control point with 5/6 rule (eq. 87)
            C_v((length(C_v)/2)+0.5) = (5/6)*C_v((length(C_v)/2)+0.5) + (1/6)*C_v((length(C_v)/2)+1.5);
            x0_l((length(x0_l)/2)+0.5) = (5/6)*x0_l((length(x0_l)/2)+0.5) + (1/6)*x0_l((length(x0_l)/2)+1.5);
            
            %Using two chordwise pivitol points at 90.45% and 34.55% chord (eq 28)         

            % X of aft pivitol point

            xAft = x0_l + 0.9045.*C_v;


            % X of forward pivitol point

            xFord = x0_l + 0.3445.*C_v;

            % Correlation factor (eq 85)

            cor = (abs(etaf(1:end-2) - etaf(3:end))./(m+1)).*cos(v.*pi./(m+1));
            cor = cor.*(span./(2.*C_v)).^2;

            % Calculate induction factors (eq 86)

            id_ = 1.974 + 0.623.*cor;
            idd_ = 1.405 + 1.009.*cor;

            jd_ = 0.285 - 4.805.*cor;
            jdd_ = 3.170 + 5.758.*cor;

            % Factors (denominator eq 110)

            d = id_.*jdd_ - idd_.*jd_;

            %Calcualte coefficiend l', l'', m' and m'' using (eq 110)
            ld = jdd_./d;
            ldd = jd_./d;

            md = idd_./d;
            mdd = id_./d;

            % Calculate non-dimensoinal X & Y

            Xd = repmat(x0_l,m,1);
            Xdd = Xd;

            Xd = (xAft' - Xd)./C_v;
            Xdd = (xFord' - Xdd)./C_v;

            Y = abs(y_v - y_v')./C_v;

            % Integrate for id & idd (Appendix 3)

            id = zeros(m);
            idd = zeros(m);
            jd = zeros(m);
            jdd = zeros(m);

            phi = linspace(0,pi,25); %Increasing this number increases computation time but improves accuracy

            c_phi = cos(phi);

            Ad = (Xd-.25)./Y;
            Add = (Xdd-.25)./Y;

            for ii = 1:m
                for jj = 1:m
                    Zd = ((1+c_phi).*(Ad(ii,jj) + ((2.*c_phi - 1)./(4.*Y(ii,jj)))))./sqrt(1 + (Ad(ii,jj) + ((2.*c_phi - 1)./(4.*Y(ii,jj)))).^2);
                    Zdd = ((1+c_phi).*(Add(ii,jj) + ((2.*c_phi - 1)./(4.*Y(ii,jj)))))./sqrt(1 + (Add(ii,jj) + ((2.*c_phi - 1)./(4.*Y(ii,jj)))).^2);

                    id(ii,jj) = 1 + (1/pi)*trapz(phi,Zd);
                    idd(ii,jj) = 1 + (1/pi)*trapz(phi,Zdd);

                    Zd = ((2.*c_phi.^2 + c_phi - 1).*(2*Xd(ii,jj) - 1 + c_phi))./sqrt((2*Xd(ii,jj) - 1 + c_phi).^2 + 4*Y(ii,jj)^2);
                    Zdd = ((2.*c_phi.^2 + c_phi - 1).*(2*Xdd(ii,jj) - 1 + c_phi))./sqrt((2*Xdd(ii,jj) - 1 + c_phi).^2 + 4*Y(ii,jj)^2);

                    jd(ii,jj) = (4/pi)*trapz(phi,Zd);
                    jdd(ii,jj) = (4/pi)*trapz(phi,Zdd);
                end
            end
            
            id(isnan(id)) = 0;
            idd(isnan(idd)) = 0;

            % a_vn matrix eq(121 & 122)

            a = (4.*sin(theta_v').*sin(theta_v))./((m+1)^2.*(eta'-eta).^2);

            idx = mod(v'-v, 2) == 0;
            a(idx) = 0;

            a(1:m+1:end) = (4/(m+1))*sin(theta_v);
            
            % for drag, calculation of induced angle of attack (dq. 143,
            % 144)
            
            b = repmat(sin(theta_v),length(theta_v),1);
            b = b./(cos(theta_v)-cos(theta_v)').^2;
            b = b./(m+1);
            
            idx = abs(repmat(1:m,m,1) - repmat((1:m)',1,m));
            idx = mod(idx,2) == 0;
            
            b(idx) = 0;
            
            b(1:m+1:end) = (m+1)./(4.*sin(theta_v));
            
            aDrag = repmat(cos(((1:m)'.*pi)./(m+1)),1,m);
            aDrag = ((4.*b)./(m+1)).*aDrag;

            % Build linear system for solution (eq. 115).

            B = a.*(ld.*id - ldd.*idd);
            C = a.*(ld.*jd - ldd.*jdd);
            D = -a.*(md.*id - mdd.*idd);
            E = -a.*(md.*jd - mdd.*jdd);

            %Assemble B, C, D, E into solution matrix
            CS = [B C; D E];
            
            %Leading diagonal is -1
            CS(1:size(CS,2)+1:end) = -1;
            
            %Update class variables
            obj.span = span;
            obj.AR = AR;
            obj.sweep = sweep;
            obj.taper = taper;
            obj.v = v;
            obj.eta = eta;
            obj.theta = theta_v;
            obj.Cv = C_v;
            obj.Cr = Cr;
            obj.S = S;
            obj.calc.CS = CS;
            obj.calc.CS_inv = CS^-1; %Save inverse to save computational time later
            obj.calc.a = a;
            obj.calc.ld = ld;
            obj.calc.ldd = ldd;
            obj.calc.md = md;
            obj.calc.mdd = mdd;
            obj.c25 = x0_l + 0.25.* C_v;
            obj.calc.aDrag = aDrag;
            obj.calc.b = b;
            
            obj.MAC = (2/3)*(Cr)*((1+taper^2+taper)/(1+taper));
            
        end
        
        function obj = setControlClass(obj, ControlClass, L, X, varargin)
           
           if ~ismethod(ControlClass, 'initialise')
               error('Control class must contain an initialise method');
           end
           
           if ~ismethod(ControlClass, 'Cl2Delta')
               warning('Control class does not contain a Cl2Delta method');
           end
           
           if ~ismethod(ControlClass, 'Delta2Cl')
               error('Control class must contain a Delta2Cl method');
           end
           
           if ~ismethod(ControlClass, 'Yaw')
               warning('Control class does not contain a Yaw method');
           end
           
           if ~ismethod(ControlClass, 'Pitch')
               warning('Control class does not contain a Pitch method');
           end
           
           if ~ismethod(ControlClass, 'Roll')
               warning('Control class does not contain a Roll method');
           end
           
           obj.ctrl = initialise(ControlClass, L, X, varargin{:});
             
           
           
        end
        
        function obj = solveAll(obj)
            
            %Solve for gamma and mu (eq 114)
            RHS1 = -obj.calc.a(1:obj.m+1:end).*(obj.calc.ld.*1 - obj.calc.ldd.*1);
            RHS2 = obj.calc.a(1:obj.m+1:end).*(obj.calc.md.*1 - obj.calc.mdd.*1);

            RHS = [RHS1'; RHS2'];

            SOL = obj.calc.CS_inv*RHS;

            %Seperate gamma and mu from solution
            obj.coefs.gamma1 = SOL(1:end/2)';
            obj.coefs.mu1 = SOL(end/2+1:end)';
            
            %As we have evaluated for alpha = 1 Cl_alpha = Cl (Eq. 132)
            obj.coefs.Cl_alpha = ((pi*obj.AR)/(obj.m+1)).*sum(obj.coefs.gamma1.*sin(obj.theta));
            
            %And we caalcualte the centre of pressure from (eq 141)
            x = obj.c25./(2*obj.span);
            
            obj.coefs.xNP = (2*obj.span).*sum((obj.coefs.gamma1.*x - obj.coefs.mu1.*(2.*obj.Cv./obj.span)).*sin(obj.theta))./sum(obj.coefs.gamma1.*sin(obj.theta));
            
            %Set CG to centre of pressure (close to neutral point)
            oCG = obj.CG(1);
%             x = x./obj.MAC;            
            
            obj.CG(1) = obj.coefs.xNP;
            
            %Loop over range of alpha for drag at pitching moment slope
            alphaR = -10:10;
            
            v = (1:obj.m) - 0.5*(obj.m - 1);
                      
            
            for ii = 1:length(alphaR)
                
                g1 = deg2rad(alphaR(ii)).*obj.coefs.gamma1;
                
                b = obj.calc.b;
                b(1:obj.m+1:end) = 0;
                alphai = b*g1';
                b = obj.calc.b;
                b = b(1:obj.m+1:end);
                alphai = b.*g1 - alphai';
                
                cv = cos((v.*pi)./(obj.m+1));
                
                cdit = g1.*alphai;

                cdi(ii) = ((pi*obj.AR)./(obj.m+1)).*sum(cdit.*cv);
                
                obj = obj.solve(alphaR(ii));
                Cm(ii) = obj.sol.Cmm;
            end
            
            %Fitting constant for induced drag
            k = polyfit((deg2rad(alphaR).*obj.coefs.Cl_alpha).^2, cdi, 1);
            
            obj.coefs.k = k(1);
            
            % keep looping for pitching moment slope until we find neutral
            % point
            Cma = polyfit(alphaR, Cm, 1);
            Cma = Cma(1);
            dCm = Cma;
            
            CGs = obj.CG(1);
            
            obj.CG(1) = 0;
            
            xNP = obj.coefs.xNP;
            
            while abs(dCm) > 1e-10
                for ii = 1:length(alphaR)
                    obj = obj.solve(alphaR(ii));
                    Cm(ii) = obj.sol.Cmm;
                end
                
                dCm = polyfit(alphaR, Cm, 1);
                dCm = dCm(1);
                Cma(end+1) = dCm;
                CGs(end+1) = obj.CG(1);
                
                [Cma, idx] = unique(Cma);
                CGs = CGs(idx);
                
                xNP = obj.CG(1);
                
                obj.CG(1) = interp1(Cma, CGs, 0, 'linear', 'extrap');
            end
            
            obj.coefs.xNP = xNP;
            obj.CG(1) = oCG;            
            
        end
        
        function obj = solve(obj, alpha, def)
            
            % Geometric angle of attack
            ad = deg2rad(alpha);
            add = deg2rad(alpha);
            
            % Update changes in effective angle of attack from control
            % surface deflections
            
            if nargin >= 3
                obj = obj.Delta2Dalpha(def);
                
                ad = ad - obj.dad;
                add = add - obj.dadd;
               
            end
            
            %Solve for gamma and mu (eq 114)
            RHS1 = -obj.calc.a(1:obj.m+1:end).*(obj.calc.ld.*(ad) - obj.calc.ldd.*(add));
            RHS2 = obj.calc.a(1:obj.m+1:end).*(obj.calc.md.*(ad) - obj.calc.mdd.*(add));

            RHS = [RHS1'; RHS2'];

            SOL = obj.calc.CS_inv*RHS;

            %Seperate gamma and mu from solution
            obj.sol.gamma = SOL(1:end/2);
            obj.sol.mu = SOL(end/2+1:end);
            
            %Calculate lift coefficient (Eq. 132)
            obj.sol.Cl = ((pi*obj.AR)/(obj.m+1))*sum(obj.sol.gamma'.*sin(obj.theta));
            
            %Calculate drag
            
            v = (1:obj.m) - 0.5*(obj.m +1);
            
            %Non-dimensional circulation
            g1 = obj.sol.gamma';
            
            %Calculate induced angle of attack
            b = obj.calc.b;
            b(1:obj.m+1:end) = 0;
            alphai = b*g1';
            b = obj.calc.b;
            b = b(1:obj.m+1:end);
            alphai = b.*g1 - alphai';

            cv = cos((v.*pi)./(obj.m+1));
            
            %Induced drag is lift multiplied by induced angle
            cdit = g1.*alphai;
            cdi = ((pi*obj.AR)./(obj.m+1)).*sum(cdit.*cv);
            
            obj.sol.Cd_dist = ((pi*obj.AR)./(obj.m+1)).*cdit.*cv;
            obj.sol.Cd = cdi;
            
            %% Calculate moments (in wind axis)
            
            x = obj.c25 - obj.CG(1);
            x = x./(2*obj.span);
%             x = zeros(size(x));

            % Calculate moments

            if obj.beta == 0
                %Pitching moment (eq. 140)
                obj.sol.Cmm = ((pi*obj.AR^2)/(2*(obj.m+1)))*sum((obj.sol.mu'.*(2.*obj.Cv./obj.span) - obj.sol.gamma'.*x).*sin(obj.theta));
                
                %Rolling moment (eq. 135)
                obj.sol.Clm = -((pi*obj.AR)/(obj.m+1))*sum(obj.sol.gamma'.*sin(obj.theta).*obj.eta.*0.5);
                
                %Yawing moment (Integrate drag distribution multiplied by moment arm)
                obj.sol.Cnm = ((pi*obj.AR)./(obj.m+1)).*sum(cdit.*cv.*obj.eta.*0.5);
            
            else
                
                %Calculate moments as above
                Cmm = ((pi*obj.AR^2)/(2*(obj.m+1)))*sum((obj.sol.mu'.*(2.*obj.Cv./obj.span) - obj.sol.gamma'.*x).*sin(obj.theta));

                y = (obj.eta - obj.CG(2)./obj.beta_span).*0.5;
                
                Clm = -((pi*obj.AR)/(obj.m+1))*sum(obj.sol.gamma'.*sin(obj.theta).*y);
                Cnm = ((pi*obj.AR)./(obj.m+1)).*sum(cdit.*cv.*y);
                
                %Change from wind axis to body axis
                
                moms = obj.beta_rot^-1*[Cmm; Clm];
                
                obj.sol.Cmm = moms(1);
                obj.sol.Clm = moms(2);
                obj.sol.Cnm = Cnm;
                
            end
            
            %Add profile drag contributions
            if nargin >= 3
                obj.sol.Cnm = obj.sol.Cnm + obj.ctrl.Yaw(def);
                obj.sol.Cd = obj.sol.Cd + sum(obj.ctrl.Drag(def));
            end
            
        end
        
        function obj = Delta2Dalpha(obj, def)
            %This function calcualates the effective angle of attack change
            %due to control surface deflections
            
            %Angle of attack at front and aft pivitol points
            ad = zeros(1, obj.m);
            add = zeros(1, obj.m);
            
            %Lift and pitching moment due to control suface deflection
            %2D values
            Cl = obj.ctrl.Delta2Cl(def);
            Cm = obj.ctrl.Pitch(def);
            
            %Loop of each control surface
            for ii = 1:obj.ctrl.nFlaps*2
                %Find regions influenced by control surface
                % i.e. prt <= eta <= stbd
                prt = obj.ctrl.X(ii)-obj.ctrl.L(ii);
                stbd = obj.ctrl.X(ii);
                
                if obj.beta ~= 0
                    prt = prt*0.5*obj.span;
                    prt = [prt; interp1(obj.beta0Aft(1,:), obj.beta0Aft(2,:), prt)];
                    
                    stbd = stbd*0.5*obj.span;
                    stbd = [stbd; interp1(obj.beta0Aft(1,:), obj.beta0Aft(2,:), stbd)];
                    
                    prt = obj.beta_rot*prt - obj.beta_cen;
                    stbd = obj.beta_rot*stbd - obj.beta_cen;
                    
                    prt = prt(1)/(obj.beta_span*0.5);
                    stbd = stbd(1)/(obj.beta_span*0.5);
                    
                end
                               
                idx = obj.eta >= prt & obj.eta <= stbd;
                
                %Change in angle of attack at pitivol points from Appendix
                %2
                
                ad(idx) = Cl(ii)/(2*pi) - (Cm(ii)/pi)*(sqrt(5) - 1);
                add(idx) = Cl(ii)/(2*pi) + (Cm(ii)/pi)*(sqrt(5) + 1);
            end
            
            obj.dad = -ad;
            obj.dadd = -add;
            
            obj.dCls = Cl;
            obj.dCms = Cm;
            
        end
        
    end
end

