%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A novel control allocation method for yaw control of             %
% tailless aircraft                                                %
% Thomas R Shearwood, Mostafa R A Nabawy, William J Crowther       %
% and Clyde Warsop                                                 %
%                                                                  %
% Example code to find the control mode shapes for a               %
% user specified trapezoidal planform.                             %
%                                                                  %
% This code requires the Statistics and Machine Learning Toolbox   %
% and Parallel Computing Toolbox.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clearvars;
close all;

%% Inputs

%Flight condition
alpha = 2; %Angle of attack

%Aircraft geometry - Trapezoidal planform
aircraft.span = 1; %Wing span [m]
aircraft.sweep = 30; %Quater chord sweep [m]
aircraft.AR = 3; %Aspect ratio []
aircraft.taper = 0.2; %Taper ratio []

aircraft.SM = 0.05; %Static margin in terms of MAC []

%Control surface geometry
control.cfc = 0.2; %Ratio of chord of control surface to wing chord (local)
control.L = [0.2, 0.2, 0.2]; %Length of control surface normalised by 0.5*span
control.X = [0.9, 0.7, 0.5]; %Outer position of control surface normalised by 0.5*span


%% Initalise models

%Initalisation performed in modeShapes class
ms = modeShapes(aircraft, control);

%% Generate inputs for calculation of modeshapes

ms = ms.generateHCS(1e5);

%% Generate mode shapes

%Evaluate aerodynamic forces and moments for each input generated in last
%step (evaluates using parfor to improve speed of execution)

ms = ms.evalCoefs(alpha);

%Fit response surface to aerodynamic forces and moments
ms = ms.calcRS();

%Calculate modeshapes from response surface
ms = ms.calcMS();

%% Output the modeshapes

nMS = size(ms.n,2);
disp(['This aircraft has ' num2str(nMS) ' modeshapes']);
disp('');

fstr = "";
for ii = 1:size(ms.n,1)
    fstr = fstr + "%.2f, "; 
end

fstr = char(fstr);

fstr = fstr(1:end-2);

for ii = 1:nMS
    disp(['Mode ' num2str(ii) ':']);
    fprintf([fstr '\n\n'], ms.n(:,ii));
end