classdef modeShapes
    %MODESHAPES Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        LS
        
        aircraft
        control
        
        X %Inputs from hypercube
        
        eval %Outputs with control deflections X
        
        RS %Response surface for aerodynamic outputs(X)
        
        n
    end
    
    methods
        function obj = modeShapes(aircraft, control)
            
            %Initialise lifting surface model with 161 spanwise points
            %(should be an odd number of points)
            obj.LS = LiftingSurface(161);
            
            %Save definition of aircraft and control surface geometry
            obj.aircraft = aircraft;
            obj.control = control;
            
            %Specify aircraft geometry and construct
            %matricies required for aerodynamic calculatios
            obj.LS = obj.LS.setGeometry(aircraft.span, aircraft.AR, aircraft.sweep, aircraft.taper);

            %Specify control surface geometry
            %This can be replaced with another class for 
            %different control methods if desired
            obj.LS = obj.LS.setControlClass(thinAirfoilTheoryPD(control.cfc), control.L, control.X, aircraft.sweep, aircraft.span, obj.LS.Cr, aircraft.taper, obj.LS.S);
        
            %Solve for aerodynamic coefficients
            obj.LS = obj.LS.solveAll();
            
            %Set aircraft Centre of Gravity using static margin specified.
            xNP = obj.LS.coefs.xNP;
            MAC = obj.LS.MAC;

            obj.LS = obj.LS.setCG(xNP-aircraft.SM*MAC);
        end
        
        function obj = generateHCS(obj, samples)

            defR = deg2rad([-10 10]);

            nFlaps = 2*length(obj.control.X);

            Xrand = rand(samples, nFlaps);

            X = linspace(defR(1), defR(2), samples)';
            X = repmat(X,1,nFlaps);

            Xrand = Xrand.*(X(2,1)-X(1,1));

            X = X+Xrand;

            idx = randperm(samples)';

            for ii = 2:nFlaps
                idx = [idx randperm(samples)'];
            end

            obj.X = X(idx);
        end
        
        function obj = evalCoefs(obj, alpha)

            Cl = zeros(size(obj.X,1),1);
            Cd = zeros(size(obj.X,1),1);
            Clm = zeros(size(obj.X,1),1);
            Cmm = zeros(size(obj.X,1),1);
            Cnm = zeros(size(obj.X,1),1);
            parfor ii = 1:size(obj.X,1)

                K = obj.LS;

                K = K.solve(alpha, obj.X(ii,:));

                Cl(ii) = K.sol.Cl;
                Cd(ii) = K.sol.Cd;
                Clm(ii) = K.sol.Clm;
                Cmm(ii) = K.sol.Cmm;
                Cnm(ii) = K.sol.Cnm;
            end
            
            obj.eval = struct();
            obj.eval.Cl = Cl;
            obj.eval.Cd = Cd;
            obj.eval.Clm = Clm;
            obj.eval.Cmm = Cmm;
            obj.eval.Cnm = Cnm;
            
        end
        
        function obj = calcRS(obj)
            %Fitting the response surface uses a lot of memory
            %To avoid exceeding the available memory we remove
            %points which yeild large pitching and rolling moments
            %as these are not significant in calculating the mode shapes
            
            % Remove points with large pitching moment
            % i.e. Cm > 0.005

            idx = abs(obj.eval.Cmm) < 5e-3;

            Cl = obj.eval.Cl(idx);
            Cd = obj.eval.Cd(idx);
            Clm = obj.eval.Clm(idx);
            Cnm = obj.eval.Cnm(idx);
            Cmm = obj.eval.Cmm(idx);
            X = obj.X(idx,:);

            % Remove points with large rolling moment
            % i.e. Cl > 0.005

            idx = abs(Clm) < 5e-3;

            Cl = Cl(idx);
            Cd = Cd(idx);
            Clm = Clm(idx);
            Cnm = Cnm(idx);
            Cmm = Cmm(idx);
            X = X(idx,:);
            
            %Fit surfaces
            obj.RS.rCl = regstats(Cl, X, 'linear');
            obj.RS.rCd = regstats(Cd, X, 'quadratic');
            obj.RS.rCmm = regstats(Cmm, X, 'linear');
            obj.RS.rClm = regstats(Clm, X, 'linear');
            obj.RS.rCnm = regstats(Cnm, X, 'quadratic');
            
        end
        
        function obj = calcMS(obj)
            A = [obj.RS.rCl.beta(2:end)'; obj.RS.rCmm.beta(2:end)'; obj.RS.rClm.beta(2:end)'];

            n = null(A);
            obj.n = n./max(abs(n),[],1);
        end
    end
end

