classdef thinAirfoilTheoryPD
    %THINAIRFOILTHEORY Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        phi
        nFlaps
        cfc
        
        L
        X
        
        arm
        muFlap
        Cd0_10
    end
    
    methods
        function obj = thinAirfoilTheoryPD(cf_c)
            %THINAIRFOILTHEORY Construct an instance of this class
            %   Calculate phi for thin airfoil theory and save
            obj.phi = acos(2*cf_c-1);
            
            obj.cfc = cf_c;
            
            
        end
        
        function obj = initialise(obj, L, X, sweep, span, Cr, taper, S)
            %Save number of control surfaces on each wing
            obj.nFlaps = length(L);
            
            %Create vector of control surface size and position from port
            %to starboard
            [L, idx] = sort(L);
            X = X(idx);
            obj.L = [fliplr(L) L];
            obj.X = [-fliplr(X)+L X];
            
            %Profile drag coefficient when control surface deflected by 10
            %degrees, Curve fit from figure 1a ESDU 06014
            Cd0_10 = 0.0286.*obj.cfc.^2 + 0.0207.*obj.cfc;
            obj.Cd0_10 = Cd0_10.*cosd(sweep);

            %Calculate mu from ESDU F.02.01.07
            h = obj.L.*0.5.*span;
            eta1 = obj.X;
            eta0 = eta1 - obj.L;
            
            %Moment arm for yawing moment due to profile drag
            obj.arm = 0.25.*(eta0+eta1);

            C1 = Cr - (1-taper).*Cr.*abs(eta1);
            C0 = Cr - (1-taper).*Cr.*abs(eta0);

            Aflap = 0.5.*h.*(C0+C1);
            obj.muFlap = Aflap./S;
        end
        
        function Cl = Delta2Cl(obj, def)
            %Caclulate change in sectional lift using thin airfoil theory
            A = zeros(3,length(def));
            A(1,:) = (obj.phi/pi-1).*def;
            A(2,:) = (2.*sin(obj.phi)/pi).*def;
            A(3,:) = (sin(2.*obj.phi)/pi).*def;
            
            Cl = pi.*(A(2,:) - 2*A(1,:));
        end
        
        function Cm = Pitch(obj, def)
            %Caclulate change in sectional pitching moment
            %using thin airfoil theory
            A = zeros(3,length(def));
            A(1,:) = (obj.phi/pi-1).*def;
            A(2,:) = (2.*sin(obj.phi)/pi).*def;
            A(3,:) = (sin(2.*obj.phi)/pi).*def;
            
            Cm = -0.25.*pi.*(A(2,:) - A(3,:));
        end
        
        function Cd = Drag(obj, def)
            %Change in sectional profile drag takes the form
            % Cd = (delta/10)^2*Cd_(10) where angles are in degrees
            % Converted to full span using ESDU F.02.01.07
            Cd = sum(def.^2.*obj.Cd0_10.*obj.muFlap./deg2rad(10).^2);
        end
        
        function Cn = Yaw(obj, def)
            if size(obj.arm,1) ~= size(def,1)
                def = def';
            end
            %Change in sectional profile drag takes the form
            % Cd = (delta/10)^2*Cd_(10) where angles are in degrees
            % Converted to full span using ESDU F.02.01.07
            %Yawing moment is drag multiplied by moment arm
            Cn = sum(obj.arm.*def.^2.*obj.Cd0_10.*obj.muFlap./deg2rad(10).^2);
        end
    end
end

